@@ -265,9 +265,9 @@ static ortp_socket_t create_and_bind(const char *addr, int *port, int *sock_fami
 	if (ortp_WSARecvMsg == NULL) {
 		GUID guid = WSAID_WSARECVMSG;
 		DWORD bytes_returned;
-		if (WSAIoctl(sock, SIO_GET_EXTENSION_FUNCTION_POINTER, &guid, sizeof(guid),
+		if (  WSAIoctl(sock, SIO_GET_EXTENSION_FUNCTION_POINTER, &guid, sizeof(guid),
 			&ortp_WSARecvMsg, sizeof(ortp_WSARecvMsg), &bytes_returned, NULL, NULL) == SOCKET_ERROR) {
-			ortp_warning("WSARecvMsg function not found.");
+			ortp_warning("WSARecvMsg function not found [%d].", WSAGetLastError());
 		}
 	}
 #endif
@@ -662,7 +662,7 @@ int rtp_session_set_dscp(RtpSession *session, int dscp){
 	if (session->rtp.gs.socket == (ortp_socket_t)-1) return 0;
 
 #if (_WIN32_WINNT >= 0x0600) && defined(ORTP_WINDOWS_DESKTOP)
-#ifndef ENABLE_MICROSOFT_STORE_APP
+#if !defined(ENABLE_MICROSOFT_STORE_APP) && !defined(ORTP_WINDOWS_UWP)
 	ortp_message("check OS support for qwave.lib");
 	if (IsWindowsVistaOrGreater()) {
 		if (session->dscp==0)
@@ -736,7 +736,7 @@ int rtp_session_set_dscp(RtpSession *session, int dscp){
 			}
 		}
 #if (_WIN32_WINNT >= 0x0600) && defined(ORTP_WINDOWS_DESKTOP)
-#ifndef ENABLE_MICROSOFT_STORE_APP
+#if !defined(ENABLE_MICROSOFT_STORE_APP) && !defined(ORTP_WINDOWS_UWP)
 	}
 #endif // ENABLE_MICROSOFT_STORE_APP
 #endif
@@ -1017,7 +1017,7 @@ void rtp_session_flush_sockets(RtpSession *session){
 
 #if defined(_WIN32) || defined(_WIN32_WCE)
 #define MAX_BUF 64
-static int rtp_sendmsg(int sock, mblk_t *m, const struct sockaddr *rem_addr, socklen_t addr_len) {
+static int rtp_sendmsg(ortp_socket_t sock, mblk_t *m, const struct sockaddr *rem_addr, socklen_t addr_len) {
 	WSAMSG msg = {0};
 	WSABUF wsabuf[MAX_BUF];
 	DWORD dwBytes = 0;
@@ -1030,10 +1030,12 @@ static int rtp_sendmsg(int sock, mblk_t *m, const struct sockaddr *rem_addr, soc
 	struct sockaddr_storage v4, v6Mapped;
 	socklen_t v4Len=0, v6MappedLen=0;
 	bool_t useV4 = FALSE;
+	struct sockaddr remote_address = {0};
+	struct addrinfo* feed_server = NULL;
 
 	for (wsabufLen = 0; wsabufLen < MAX_BUF && mTrack != NULL; mTrack = mTrack->b_cont, ++wsabufLen) {
+		wsabuf[wsabufLen].len = (ULONG)(mTrack->b_wptr - mTrack->b_rptr);
 		wsabuf[wsabufLen].buf = mTrack->b_rptr;
-		wsabuf[wsabufLen].len = mTrack->b_wptr - mTrack->b_rptr;
 	}
 	msg.lpBuffers = wsabuf; // contents
 	msg.dwBufferCount = wsabufLen;
@@ -1045,8 +1047,14 @@ static int rtp_sendmsg(int sock, mblk_t *m, const struct sockaddr *rem_addr, soc
 		}
 		ortp_error("Too long msgb (%i fragments) , didn't fit into iov, end discarded.", MAX_BUF + count);
 	}
-	msg.name = (SOCKADDR *)rem_addr;
-	msg.namelen = addr_len;
+	if( addr_len == 0){
+		// Get the local host information
+		msg.namelen = 0;
+		msg.name = NULL;
+	}else{
+		msg.namelen = addr_len;
+		msg.name = (SOCKADDR *)rem_addr;
+	}
 	msg.Control.buf = ctrlBuffer;
 	msg.Control.len = sizeof(ctrlBuffer);
 	cmsg = WSA_CMSG_FIRSTHDR(&msg);
@@ -1085,22 +1093,26 @@ static int rtp_sendmsg(int sock, mblk_t *m, const struct sockaddr *rem_addr, soc
 	msg.Control.len = controlSize;
 	if (controlSize == 0)
 		msg.Control.buf = NULL;
-	error = WSASendMsg(sock, &msg, 0, &dwBytes, NULL, NULL);
-	 if( error == SOCKET_ERROR && controlSize != 0){
+	error = WSASendMsg((SOCKET)sock, &msg, 0, &dwBytes, NULL, NULL);
+	if( error == SOCKET_ERROR && controlSize != 0){// Debug note : if unexpected WSAEFAULT, check the location of memory allocation of msg. It must be on ortp addresses space.
 		int errorCode = WSAGetLastError();
 		if( errorCode == WSAEINVAL || errorCode==WSAENETUNREACH || errorCode==WSAEFAULT)
 		{
 			msg.Control.len = 0;
 			msg.Control.buf = NULL;
-			error = WSASendMsg(sock, &msg, 0, &dwBytes, NULL, NULL);
+			error = WSASendMsg((SOCKET)sock, &msg, 0, &dwBytes, NULL, NULL);
 		}
-	 }
-	return error;
+	}
+	mTrack = m;
+	if(error >= 0)
+		return dwBytes;// Return the bytes that have been sent
+	else
+		return error;
 }
 #else
 #ifdef USE_SENDMSG
 #define MAX_IOV 64
-static int rtp_sendmsg(int sock,mblk_t *m, const struct sockaddr *rem_addr, socklen_t addr_len){
+static int rtp_sendmsg(ortp_socket_t sock,mblk_t *m, const struct sockaddr *rem_addr, socklen_t addr_len){
 	struct msghdr msg;
 	struct iovec iov[MAX_IOV];
 	int iovlen;
@@ -1210,11 +1222,11 @@ static int rtp_sendmsg(int sock,mblk_t *m, const struct sockaddr *rem_addr, sock
 	msg.msg_controllen = controlSize;
 	if( controlSize==0) // Have to reset msg_control to NULL as msg_controllen is not sufficient on some platforms
 		msg.msg_control = NULL;
-	error = sendmsg(sock,&msg,0);
+	error = sendmsg((int)sock,&msg,0);
 	if( error == -1 && controlSize != 0 && (errno == EINVAL || errno==ENETUNREACH || errno==EFAULT)) {
 		msg.msg_controllen =0;
 		msg.msg_control = NULL;
-		error = sendmsg(sock,&msg,0);
+		error = sendmsg((int)sock,&msg,0);
 	}
 	return error;
 }
@@ -1226,15 +1238,15 @@ ortp_socket_t rtp_session_get_socket(RtpSession *session, bool_t is_rtp){
 }
 
 int _ortp_sendto(ortp_socket_t sockfd, mblk_t *m, int flags, const struct sockaddr *destaddr, socklen_t destlen) {
-	int error;
+	int sent_bytes;
 #if defined(_WIN32) || defined(_WIN32_WCE) || defined(USE_SENDMSG)
-	error = rtp_sendmsg(sockfd, m, destaddr, destlen);
+	sent_bytes = rtp_sendmsg(sockfd, m, destaddr, destlen);
 #else
 	if (m->b_cont != NULL)
 		msgpullup(m, -1);
-	error = sendto(sockfd, (char*)m->b_rptr, (int)(m->b_wptr - m->b_rptr), 0, destaddr, destlen);
+	sent_bytes = sendto(sockfd, (char*)m->b_rptr, (int)(m->b_wptr - m->b_rptr), 0, destaddr, destlen);
 #endif
-	return error;
+	return sent_bytes;
 }
 
 int rtp_session_sendto(RtpSession *session, bool_t is_rtp, mblk_t *m, int flags, const struct sockaddr *destaddr, socklen_t destlen){
@@ -1253,7 +1265,7 @@ int rtp_session_sendto(RtpSession *session, bool_t is_rtp, mblk_t *m, int flags,
 		putq(&session->net_sim_ctx->send_q, m);
 		ortp_mutex_unlock(&session->net_sim_ctx->mutex);
 	}else{
-		ortp_socket_t sockfd = rtp_session_get_socket(session, is_rtp);
+		ortp_socket_t sockfd = rtp_session_get_socket(session, is_rtp || session->rtcp_mux);
 		if (sockfd != (ortp_socket_t)-1){
 			ret=_ortp_sendto(sockfd, m, flags, destaddr, destlen);
 		}else{
@@ -1517,7 +1529,6 @@ static int rtp_recvmsg(ortp_socket_t socket, mblk_t *msg, int flags, struct sock
 	return error;
 }
 #endif
-
 int rtp_session_rtp_recv_abstract(ortp_socket_t socket, mblk_t *msg, int flags, struct sockaddr *from, socklen_t *fromlen) {
 	int ret;
 	int bufsz = (int) (msg->b_datap->db_lim - msg->b_datap->db_base);
@@ -1526,8 +1537,8 @@ int rtp_session_rtp_recv_abstract(ortp_socket_t socket, mblk_t *msg, int flags,
 	WSAMSG msghdr = {0};
 	WSACMSGHDR *cmsghdr;
 	WSABUF data_buf;
-	DWORD bytes_received;
-
+	DWORD bytes_received=0;
+	int error = 0;
 	if (ortp_WSARecvMsg == NULL) {
 		return recvfrom(socket, (char *)msg->b_wptr, bufsz, flags, from, fromlen);
 	}
@@ -1538,8 +1549,8 @@ int rtp_session_rtp_recv_abstract(ortp_socket_t socket, mblk_t *msg, int flags,
 	}
 	data_buf.buf = (char *)msg->b_wptr;
 	data_buf.len = bufsz;
-	msghdr.lpBuffers = &data_buf;
 	msghdr.dwBufferCount = 1;
+	msghdr.lpBuffers = &data_buf;
 	msghdr.Control.buf = control;
 	msghdr.Control.len = sizeof(control);
 	msghdr.dwFlags = flags;
@@ -1612,6 +1623,7 @@ int rtp_session_rtp_recv_abstract(ortp_socket_t socket, mblk_t *msg, int flags,
 			memcpy(&msg->net_addr,from,*fromlen);
 			msg->net_addrlen = *fromlen;
 		}
+	}else{
 	}
 	return ret;
 }
@@ -1921,7 +1933,11 @@ void* rtp_session_recvfrom_async(void* obj) {
 				{
 					if (session->on_network_error.count>0){
 						rtp_signal_table_emit3(&session->on_network_error,"Error receiving RTP packet",ORTP_INT_TO_POINTER(getSocketErrorCode()));
-					}else ortp_warning("Error receiving RTP packet: %s, err num  [%i],error [%i]",getSocketError(),errnum,error);
+	#ifdef _WIN32
+					}else if(errnum!=WSAECONNRESET) ortp_warning("Error receiving RTP packet err num [%i], error [%i]: %s",errnum,error,getSocketError());// Windows spam WSAECONNRESET and is not useful
+	#else
+					}else ortp_warning("Error receiving RTP packet err num [%i], error [%i]: %s",errnum,error,getSocketError());
+	#endif
 	#if TARGET_OS_IPHONE
 					/*hack for iOS and non-working socket because of background mode*/
 					if (errnum==ENOTCONN){
@@ -1942,11 +1958,12 @@ void* rtp_session_recvfrom_async(void* obj) {
 
 int rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts) {
 	mblk_t *mp;
-
+	
 	if ((session->rtp.gs.socket==(ortp_socket_t)-1) && !rtp_session_using_transport(session, rtp)) return -1;  /*session has no sockets for the moment*/
 
 	while (1)
 	{
+		bool_t packet_is_rtp = TRUE;
 		if (!session->bundle || (session->bundle && session->is_primary)) {
 #if	defined(_WIN32) || defined(_WIN32_WCE)
 			ortp_mutex_lock(&session->rtp.winthread_lock);
@@ -1978,13 +1995,24 @@ int rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts) {
 			ortp_mutex_lock(&session->bundleq_lock);
 			mp = getq(&session->bundleq);
 			ortp_mutex_unlock(&session->bundleq_lock);
+			
+			if (mp && session->rtcp_mux){
+				/* The packet could be a RTCP one */
+				if (rtp_get_version(mp) == 2){
+					int pt = rtp_get_payload_type(mp);
+					if (pt >= 64 && pt <= 95){
+						/*this is assumed to be an RTCP packet*/
+						packet_is_rtp = FALSE;
+					}
+				}
+			}
 		}
 
 		if (mp != NULL) {
 			mp->reserved1 = user_ts;
-			rtp_session_process_incoming(session, mp, TRUE, user_ts, FALSE);
+			rtp_session_process_incoming(session, mp, packet_is_rtp, user_ts, !packet_is_rtp);
 		} else {
-			rtp_session_process_incoming(session, NULL, TRUE, user_ts, FALSE);
+			rtp_session_process_incoming(session, NULL, packet_is_rtp, user_ts, FALSE);
 			return -1;
 		}
 	}
@@ -2041,7 +2069,11 @@ int rtp_session_rtcp_recv (RtpSession * session) {
 			{
 				if (session->on_network_error.count>0){
 					rtp_signal_table_emit3(&session->on_network_error,"Error receiving RTCP packet",ORTP_INT_TO_POINTER(getSocketErrorCode()));
+#ifdef _WIN32
+				}else if(errnum!=WSAECONNRESET) ortp_warning("Error receiving RTCP packet: %s, err num  [%i],error [%i]",getSocketError(),errnum,error);// Windows spam WSAECONNRESET and is not useful
+#else
 				}else ortp_warning("Error receiving RTCP packet: %s, err num  [%i],error [%i]",getSocketError(),errnum,error);
+#endif
 #if TARGET_OS_IPHONE
 				/*hack for iOS and non-working socket because of background mode*/
 				if (errnum==ENOTCONN){
